/* goprocs.h
 *
 * adapted for xigc v3.3  1/94  =SC
 */

/* Developers:  James R. Logan Jr. and H. Lynn Beus (BYU)
   email address:  loganj@byuvax.bitnet
   (801) 378-6551

   Structures required by the go program utilities.
   This code is not machine dependent.
*/

#define GOPROCS

#define Maxstones 	      361
#define Maxarmies 	      362
#define Maxarmylibs		2
#define Maxhistory 	      600

/* Colors of board positions */
#define White 	0
#define Black 	1
#define Blank 	2
#define Lifted	3

#define REALGAME 0
#define STEPGAME 1

#define NODRAWIT 0
#define DRAWIT 1

#define NONUMBERIT 0
#define NUMBERIT 1

/* Bit encoded capture directions, values 1,2,4,8 are already used */

#define SUICIDE1 16	/* Suicide of 1 stone */
#define SUICIDE2 32	/* Suicide of more than 1 stone */
#define SUICIDE_ANY 48	/* Suicide of any stone(s) */

/* Define bit encoded rules */

#define GO_DO_CAPTURES 1
#define GO_DO_UNCAPTURES 2
#define GO_DO_SUICIDE 4
#define GO_DO_ILLEGAL 8

#define RULES (GO_DO_CAPTURES | GO_DO_UNCAPTURES)
#define GOE_RULES (GO_DO_CAPTURES | GO_DO_UNCAPTURES | GO_DO_SUICIDE)
#define STEP_RULES (GO_DO_CAPTURES | GO_DO_ILLEGAL)
#define GOE_STEP_RULES (GO_DO_CAPTURES | GO_DO_ILLEGAL | GO_DO_SUICIDE)

/* Types of play.  The sum of these types of play and the 2 stone colors 
   White and Black determine the color and type of move. */
#define Playtypes 	10
	/* g->typehistory[i] = the following	*/
#define Playstone 	0
#define Pass 		2
#define Handistone 	4
#define Blankstone	6


typedef struct {
 char color;           /* Army color */
 short
  size,    	/* army size  */
  first,   	/* first armyman's position */
  last,    	/* last  armyman's position */
  fp,      	/* forward pointer to next army */
  bp,      		/* backward pointer to last army */
  lib,     		/* number of liberties */
  friend,  	/* number of neighboring White stones, for blank armies only */
  enemy;   	/* number of enemy stones for nonblank armies */
} armystruct;



typedef struct {
 short koSpot, /* valid ko position, or -1, result of last move */
  deadWhite,   /* number of captured white stones */
  deadBlack;   /* number of captured black stones */
  char 
    koColor,   /* Color of player that can not play at ko position. */
    passCount, /* number of consecutive passes made */
    captures;  /* bit 1 is capture on the west (left)
                  bit 2 is capture on the east (right)
                  bit 3 is capture on the south (down)
                  bit 4 is capture on the north (up) */
} mstatus;


typedef struct {
 short
  hsttolst[Maxhistory],		/* index from hist position to board position */
  stonenums[Maxhistory];	/* stone numbers and symbols */

 char
  gametype[Maxhistory],		/* game stone or step stone */
  typehistory[Maxhistory];	/* type of stone: handicap or play */
} SGFstruct;


typedef struct {
 short
  neis,
  stone[4];
 unsigned char
  direction[4]; 
	/* bit 1 is neighbor on the west (left)
	   bit 2 is capture on the east (right)
	   bit 3 is capture on the south (down)
	   bit 4 is capture on the north (up) */
} neighbors;


typedef struct {
 armystruct army[Maxarmies];
 neighbors nabor[Maxstones];
 SGFstruct sgf;
 
 short 
  gametype,		/* REALGAME or STEPGAME */
  who,			/* Whose turn it is to play, Black or White */
  avail,		/* next available army number */
  maxStone,		/* size of 1-dimensional board */
  width,		/* width of 2-dimensional board */
  stoneNumber,		/* sequential number of last stone added to board */
  playNumber;		/* sequential number of last stone played on board */
                        /* playNumber is intended to count after handicap */
 char
  board[ Maxstones],	/* computer = White, opponent = Black */
  rowbrd[Maxstones],	/* translates 1-dimension row to 2-dimension */
  colbrd[Maxstones];	/* translates 1-dimension col to 2-dimension */

 short

  armymen[Maxstones],	/* men of same army have same num */
  armyflnk[Maxstones],	/* link to next army member */
  armyblnk[Maxstones],	/* link to last army member */
  brdtohst[Maxstones],	/* index from board position to history position */
  history[Maxhistory];	/* history of moves, white positive, black negative */

 unsigned long
  neibrd[Maxstones];	/* n by n bit encoded valid neighbors array */

 mstatus 
  moveStatus,			/* ko position and capture info */
  stathistory[Maxhistory];	/* history of ko positions and capture info */
} game;
